﻿using System;
using NUnit.Framework;
using Vita.Domain.Accounting;

namespace Vita.Domain.Tests.Unit.Accounting
{
    [TestFixture]
    public class RecordTransactionTest : TestFixtureBase<Account>
    {
        [Test]
        public void RecordCredit()
        {
            var date = new DateTimeOffset(2011, 3, 17, 0, 0, 0, TimeSpan.FromHours(0));
            string payer = "Acme Trading Ltd";
            string paymentMethod = "BGC";
            var amount = new Money(100);
            string notes = "Notes";
            var categories = new[] { "Salary" };
            var tags = new[] { "" };

            Run(
                given: OpenAccount,
                when: account => account.Credit(date, payer, amount, paymentMethod, notes, categories, tags),
                then: account => new[] {new AccountCredited(account.Id, date, payer, amount.Invert(), paymentMethod, notes, categories, tags)}
                );
        }

        [Test]
        public void RecordCreditOfNegativeAmountShouldFail()
        {
            var date = new DateTimeOffset(2011, 3, 17, 0, 0, 0, TimeSpan.FromHours(0));
            string payer = "Acme Trading Ltd";
            string paymentMethod = "BGC";
            var amount = new Money(-100);
            string notes = "Notes";
            var categories = new[] { "Salary" };
            var tags = new[] { "" };

            Run(
                given: OpenAccount,
                when: account => account.Credit(date, payer, amount, paymentMethod, notes, categories, tags),
                expectException: typeof(ArgumentException)
                );
        }

        [Test]
        public void RecordDebit()
        {
            var date = new DateTimeOffset(2011, 3, 17, 0, 0, 0, TimeSpan.FromHours(0));
            string payee = "Acme Trading Ltd";
            string paymentMethod = "BGC";
            var amount = new Money(100);
            string notes = "Notes";
            var categories = new[] { "Salary" };
            var tags = new[] { "" };

            Run(
                given: OpenAccount,
                when: account => account.Debit(date, payee, amount, paymentMethod, notes, categories, tags),
                then: account => new[] { new AccountDebited(account.Id, date, payee, amount, paymentMethod, notes, categories, tags) }
                );
        }

        [Test]
        public void RecordDebitOfNegativeAmountShouldFail()
        {
            var date = new DateTimeOffset(2011, 3, 17, 0, 0, 0, TimeSpan.FromHours(0));
            string payee = "Acme Trading Ltd";
            string paymentMethod = "BGC";
            var amount = new Money(-100);
            string notes = "Notes";
            var categories = new[] { "Salary" };
            var tags = new[] { "" };

            Run(
                given: OpenAccount,
                when: account => account.Debit(date, payee, amount, paymentMethod, notes, categories, tags),
                expectException: typeof(ArgumentException)
                );
        }

        private Account OpenAccount()
        {
            Guid accountId = GuidProvider.Current.NewId();
            const AccountType accountType = AccountType.Asset;
            Guid accountHolderId = GuidProvider.Current.NewId();
            const string accountHolderName = "AccountHolderName";
            const string accountName = "AccountName";
            var openingBalance = new Money((decimal)123.00);

            var account = new Account();
            account.Open(accountId, accountType, accountHolderId, accountHolderName, accountName, openingBalance);
            return account;
        }
    }
}