﻿using System;
using Regalo.Core;

namespace Vita.Domain.Accounting
{
    public class Account : AggregateRoot
    {
        private IAccountBalanceChangeStrategy _accountBalanceChangeStrategy;

        public void Open(Guid id, AccountType accountType, Guid accountHolderId, string accountHolderName, string accountName, Money openingBalance)
        {
            if (accountHolderId   == null) throw new ArgumentNullException("accountHolderId");
            if (accountHolderName == null) throw new ArgumentNullException("accountHolderName");
            if (accountName       == null) throw new ArgumentNullException("accountName");
            if (openingBalance    == null) throw new ArgumentNullException("openingBalance");
            
            if (SelectBalanceChangeStrategy(accountType) == null) 
            {
                throw new ArgumentException(string.Format("Account type {0} is not supported.", accountType), "accountType");
            }

            Record(new AccountOpened(id, accountType, accountHolderId, accountHolderName, accountName, openingBalance));
        }

        private void Apply(AccountOpened evt)
        {
            Id = evt.AggregateId;

            _accountBalanceChangeStrategy = SelectBalanceChangeStrategy(evt.AccountType);
        }

        public void Credit(DateTimeOffset date, string payer, Money amount, string paymentMethod, string notes, string[] categories, string[] tags)
        {
            if (string.IsNullOrWhiteSpace(payer))         throw new ArgumentException("Payer must be supplied.");
            if (string.IsNullOrWhiteSpace(paymentMethod)) throw new ArgumentException("Payment method must be supplied.");
            if (amount.Value < 0)                         throw new ArgumentException("Cannot credit a negative amount", "amount");

            Record(new AccountCredited(Id, date, payer, _accountBalanceChangeStrategy.ApplyCredit(amount), paymentMethod, notes, categories, tags));
        }

        public void Debit(DateTimeOffset date, string payee, Money amount, string paymentMethod, string notes, string[] categories, string[] tags)
        {
            if (string.IsNullOrWhiteSpace(payee))         throw new ArgumentException("Payee must be supplied.");
            if (string.IsNullOrWhiteSpace(paymentMethod)) throw new ArgumentException("Payment method must be supplied.");
            if (amount.Value < 0)                         throw new ArgumentException("Cannot debit a negative amount", "amount");
            
            Record(new AccountDebited(Id, date, payee, _accountBalanceChangeStrategy.ApplyDebit(amount), paymentMethod, notes, categories, tags));
        }

        private IAccountBalanceChangeStrategy SelectBalanceChangeStrategy(AccountType accountType)
        {
            switch (accountType)
            {
                case AccountType.Asset:
                case AccountType.Expense:
                    return new DebitIsIncreaseBalanceStrategy();
                case AccountType.Equity:
                case AccountType.Liability:
                case AccountType.Income:
                    return new CreditIsIncreaseBalanceStrategy();
                default:
                    throw new NotSupportedException(string.Format("Account type {0} is not supported.", accountType));
            }
        }
    }
}